@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002      CodeFactory AB
- * Copyright (C) 2001-2002 Mikael Hallendal <micke@imendio.com>
- * Copyright (C) 2003      Richard Hult <richard@imendio.com>
+ * Copyright (C) 2001-2004 Imendio HB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,28 +21,33 @@
 #include <config.h>
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtkactiongroup.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtkhbox.h>
 #include <gtk/gtkhpaned.h>
 #include <gtk/gtklabel.h>
+#include <gtk/gtkmain.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkuimanager.h>
+#include <gconf/gconf-client.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-href.h>
 #include <libgnomeui/gnome-stock-icons.h>
-#include <libegg/menu/egg-menu-merge.h>
 
 #include "dh-book-tree.h"
-#include "dh-history.h"
 #include "dh-html.h"
+#include "dh-preferences.h"
 #include "dh-search.h"
 #include "dh-window.h"
 
 extern gchar *geometry;
+extern GConfClient *gconf_client;
 
 struct _DhWindowPriv {
 	DhBase         *base;
-	DhHistory      *history;
 
 	GtkWidget      *main_box;
 	GtkWidget      *menu_box;
@@ -56,10 +59,18 @@ struct _DhWindowPriv {
 
 	DhHtml         *html;
 
-	EggMenuMerge   *merge;
-	EggActionGroup *action_group;
+	GtkUIManager   *manager;
+	GtkActionGroup *action_group;
 };
 
+/* People have reported problems with the default values in GConf so I'm
+ * adding this to make sure that the window isn't started 1x1 pixels or the
+ * paned having size 0
+ */
+#define DEFAULT_WIDTH     700
+#define DEFAULT_HEIGHT    500
+#define DEFAULT_PANED_LOC 250 
+
 static void window_class_init                (DhWindowClass      *klass);
 static void window_init                      (DhWindow           *window);
  
@@ -67,8 +78,20 @@ static void window_finalize                  (GObject            *object);
 
 static void window_populate                  (DhWindow           *window);
 
-static void window_activate_action           (EggAction          *action,
+static void window_activate_quit             (GtkAction          *action,
 					      DhWindow           *window);
+static void window_activate_copy             (GtkAction          *action,
+					      DhWindow           *window);
+static void window_activate_preferences      (GtkAction          *action,
+					      DhWindow           *window);
+static void window_activate_back             (GtkAction          *action,
+					      DhWindow           *window);
+static void window_activate_forward          (GtkAction          *action,
+					      DhWindow           *window);
+static void window_activate_about            (GtkAction          *action,
+					      DhWindow           *window);
+static void window_save_state                (DhWindow           *window);
+static void window_restore_state             (DhWindow           *window);
 static void window_delete_cb                 (GtkWidget          *widget,
 					      GdkEventAny        *event,
 					      gpointer            user_data);
@@ -80,41 +103,44 @@ static void window_link_selected_cb          (GObject            *ignored,
 					      DhLink             *link,
 					      DhWindow           *window);
 
-static void window_merge_add_widget          (EggMenuMerge       *merge,
+static void window_manager_add_widget          (GtkUIManager        *manager,
 					      GtkWidget          *widget,
 					      DhWindow           *window);
-static void window_back_exists_changed_cb    (DhHistory          *history,
-					      gboolean            exists,
-					      DhWindow           *window);
-static void window_forward_exists_changed_cb (DhHistory          *history,
-					      gboolean            exists,
-					      DhWindow           *window);
 static gboolean window_key_press_event_cb    (GtkWidget          *widget,
 					      GdkEventKey        *event,
 					      DhWindow           *window);
-
-
+static void window_check_history             (DhWindow           *window);
+static void window_location_changed_cb       (DhHtml             *html,
+					      const gchar        *location,
+					      DhWindow           *window);
 
 static GtkWindowClass *parent_class = NULL;
 
-static EggActionGroupEntry actions[] = {
-	{ "StockFileMenuAction", N_("_File"), NULL, NULL, NULL, NULL, NULL },
-	{ "StockGoMenuAction", N_("_Go"), NULL, NULL, NULL, NULL, NULL },
-	{ "StockHelpMenuAction", N_("_Help"), NULL, NULL, NULL, NULL, NULL },
+static GtkActionEntry actions[] = {
+	{ "FileMenu", NULL, N_("_File") },
+	{ "EditMenu", NULL, N_("_Edit") },
+	{ "GoMenu", NULL, N_("_Go") },
+	{ "HelpMenu", NULL, N_("_Help") },
 
 	/* File menu */
-	{ "QuitAction", NULL, GTK_STOCK_QUIT, "<control>Q", NULL,
-	  G_CALLBACK (window_activate_action), NULL },
+	{ "Quit", GTK_STOCK_QUIT, NULL, "<control>Q", NULL,
+	  G_CALLBACK (window_activate_quit) },
 
+	/* Edit menu */
+	{ "Copy", GTK_STOCK_COPY, NULL, "<control>C", NULL,
+	  G_CALLBACK (window_activate_copy) },
+	{ "Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL,
+	  G_CALLBACK (window_activate_preferences) },
+	 
 	/* Go menu */
-	{ "BackAction", NULL, GTK_STOCK_GO_BACK, NULL, NULL,
-	  G_CALLBACK (window_activate_action), NULL },
-	{ "ForwardAction", NULL, GTK_STOCK_GO_FORWARD, NULL, NULL,
-	  G_CALLBACK (window_activate_action), NULL },
+	{ "Back", GTK_STOCK_GO_BACK, NULL, NULL, NULL,
+	  G_CALLBACK (window_activate_back) },
+	{ "Forward", GTK_STOCK_GO_FORWARD, NULL, NULL, NULL,
+	  G_CALLBACK (window_activate_forward) },
 
 	/* About menu */
-	{ "AboutAction", NULL, GNOME_STOCK_ABOUT, NULL, NULL,
-	  G_CALLBACK (window_activate_action), NULL }
+	{ "About", GNOME_STOCK_ABOUT, NULL, NULL, NULL,
+	  G_CALLBACK (window_activate_about) }
 };
 
 GType
@@ -147,10 +173,10 @@ static void
 window_class_init (DhWindowClass *klass)
 {
         GObjectClass *object_class;
-        
+	
         parent_class = g_type_class_peek_parent (klass);
         
-        object_class = (GObjectClass *) klass;
+        object_class = G_OBJECT_CLASS (klass);
         
         object_class->finalize = window_finalize;
 }
@@ -158,28 +184,24 @@ window_class_init (DhWindowClass *klass)
 static void
 window_init (DhWindow *window)
 {
-        DhWindowPriv *priv;
-	gint          i;
-	EggAction    *action;
+	DhWindowPriv *priv;
+	GtkAction    *action;
 	
-        priv          = g_new0 (DhWindowPriv, 1);
-	priv->history = dh_history_new ();
-
-	g_signal_connect (priv->history, "forward_exists_changed",
-			  G_CALLBACK (window_forward_exists_changed_cb),
-			  window);
-	g_signal_connect (priv->history, "back_exists_changed",
-			  G_CALLBACK (window_back_exists_changed_cb),
-			  window);
+	priv = g_new0 (DhWindowPriv, 1);
 
 	g_signal_connect (window, "key_press_event",
 			  G_CALLBACK (window_key_press_event_cb),
 			  window);
+
+	priv->html = dh_html_new ();
+	g_signal_connect (priv->html, "location-changed",
+			  G_CALLBACK (window_location_changed_cb),
+			  window);
 	
-	priv->merge = egg_menu_merge_new ();
+	priv->manager = gtk_ui_manager_new ();
 
 	gtk_window_add_accel_group (GTK_WINDOW (window),
-				    priv->merge->accel_group);
+				    gtk_ui_manager_get_accel_group (priv->manager));
 	
 	priv->main_box = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (priv->main_box);
@@ -192,33 +214,33 @@ window_init (DhWindow *window)
 	
 	gtk_container_add (GTK_CONTAINER (window), priv->main_box);
 
-	g_signal_connect (priv->merge,
+	g_signal_connect (priv->manager,
 			  "add_widget",
-			  G_CALLBACK (window_merge_add_widget),
+			  G_CALLBACK (window_manager_add_widget),
 			  window);
 
-	for (i = 0; i < G_N_ELEMENTS (actions); ++i) {
-		actions[i].user_data = window;
-	}
+	priv->action_group = gtk_action_group_new ("MainWindow");
 
-	priv->action_group = egg_action_group_new ("MainWindow");
+	gtk_action_group_set_translation_domain (priv->action_group,
+						 GETTEXT_PACKAGE);
 	
-	egg_action_group_add_actions (priv->action_group,
+	gtk_action_group_add_actions (priv->action_group,
 				      actions,
-				      G_N_ELEMENTS (actions));
+				      G_N_ELEMENTS (actions),
+				      window);
 
-	egg_menu_merge_insert_action_group (priv->merge,
+	gtk_ui_manager_insert_action_group (priv->manager,
 					    priv->action_group,
 					    0);
 
-	action = egg_action_group_get_action (priv->action_group, 
-					      "BackAction");
+	action = gtk_action_group_get_action (priv->action_group, 
+					      "Back");
 	g_object_set (action, "sensitive", FALSE, NULL);
 	
-	action = egg_action_group_get_action (priv->action_group,
-					      "ForwardAction");
+	action = gtk_action_group_get_action (priv->action_group,
+					      "Forward");
 	g_object_set (action, "sensitive", FALSE, NULL);
-	
+
         window->priv = priv;
 }
 
@@ -244,7 +266,8 @@ window_switch_page_cb (GtkWidget       *notebook,
 
 	priv = window->priv;
 
-	g_signal_handlers_block_by_func (priv->book_tree, window_link_selected_cb, window);
+	g_signal_handlers_block_by_func (priv->book_tree, 
+					 window_link_selected_cb, window);
 }
 
 static void
@@ -257,49 +280,48 @@ window_switch_page_after_cb (GtkWidget       *notebook,
 
 	priv = window->priv;
 	
-	g_signal_handlers_unblock_by_func (priv->book_tree, window_link_selected_cb, window);
+	g_signal_handlers_unblock_by_func (priv->book_tree, 
+					   window_link_selected_cb, window);
 }
 
 static void
 window_populate (DhWindow *window)
 {
         DhWindowPriv *priv;
-	GtkWidget    *html_sw;
 	GtkWidget    *frame;
 	GtkWidget    *book_tree_sw;
 	GNode        *contents_tree;
 	GList        *keywords = NULL;
 	GError       *error = NULL;
-	 
+	gint          hpaned_position;
+	
         g_return_if_fail (window != NULL);
         g_return_if_fail (DH_IS_WINDOW (window));
         
         priv = window->priv;
 	
-	egg_menu_merge_add_ui_from_file (priv->merge,
+	gtk_ui_manager_add_ui_from_file (priv->manager,
 					 DATADIR "/devhelp/ui/window.ui",
 					 &error);
+	if (error) {
+		g_warning (_("Cannot set UI: %s"), error->message);
+		g_error_free (error);
+	}
 
+	gtk_ui_manager_ensure_update (priv->manager);
+	
         priv->hpaned    = gtk_hpaned_new ();
         priv->notebook  = gtk_notebook_new ();
-	priv->html      = dh_html_new ();
-	priv->html_view = dh_html_get_widget (priv->html);
 
-	g_signal_connect (priv->notebook,
-			  "switch_page",
+	g_signal_connect (priv->notebook, "switch_page",
 			  G_CALLBACK (window_switch_page_cb),
 			  window);
 
-	g_signal_connect_after (priv->notebook,
-				"switch_page",
+	g_signal_connect_after (priv->notebook, "switch_page",
 				G_CALLBACK (window_switch_page_after_cb),
 				window);
 
-	html_sw         = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (html_sw),
-					GTK_POLICY_AUTOMATIC,
-					GTK_POLICY_AUTOMATIC);
-	book_tree_sw      = gtk_scrolled_window_new (NULL, NULL);
+	book_tree_sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (book_tree_sw),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
@@ -313,17 +335,23 @@ window_populate (DhWindow *window)
 
 	gtk_paned_add1 (GTK_PANED (priv->hpaned), frame);
 	
-/*  	gtk_container_add (GTK_CONTAINER (html_sw), priv->html_view); */
-	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (html_sw),
-					       priv->html_view);
+	priv->html_view = dh_html_get_widget (priv->html);
 	
 	frame = gtk_frame_new (NULL);
-	gtk_container_add (GTK_CONTAINER (frame), html_sw);
-	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
+	gtk_container_add (GTK_CONTAINER (frame), priv->html_view);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
 
- 	gtk_paned_add2 (GTK_PANED(priv->hpaned), frame);
+	gtk_paned_add2 (GTK_PANED(priv->hpaned), frame);
 
- 	gtk_paned_set_position (GTK_PANED (priv->hpaned), 250);
+	hpaned_position = gconf_client_get_int (gconf_client,
+						GCONF_PANED_LOCATION,
+						NULL);
+
+	if (hpaned_position <= 0) {
+		hpaned_position = DEFAULT_PANED_LOC;
+	}
+	
+ 	gtk_paned_set_position (GTK_PANED (priv->hpaned), hpaned_position);
 
 	contents_tree = dh_base_get_book_tree (priv->base);
 	keywords      = dh_base_get_keywords (priv->base);
@@ -361,86 +389,190 @@ window_populate (DhWindow *window)
 
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), 0);
 
- 	g_signal_connect_swapped (priv->html, 
-				  "uri_selected", 
-				  G_CALLBACK (window_open_url),
-				  window);
+	dh_preferences_setup_fonts ();
 }
 
 static void
-window_activate_action (EggAction *action, DhWindow *window)
+window_activate_quit (GtkAction *action, DhWindow *window)
 {
 	DhWindowPriv *priv;
-	const gchar  *name = action->name;
 	
 	g_return_if_fail (DH_IS_WINDOW (window));
 
 	priv = window->priv;
+
+	window_save_state (window);
 	
-	if (strcmp (name, "QuitAction") == 0) {
-		gtk_main_quit ();
-		/* Quit */
-	}
-	else if (strcmp (name, "BackAction") == 0) {
-		gchar *uri = dh_history_go_back (priv->history);
-		if (uri) {
-			dh_html_open_uri (priv->html, uri);
-			g_free (uri);
-		}
-	}
-	else if (strcmp (name, "ForwardAction") == 0) {
-		gchar *uri = dh_history_go_forward (priv->history);
-		if (uri) {
-			dh_html_open_uri (priv->html, uri);
-			g_free (uri);
-		}
-	}
-	else if (strcmp (name, "AboutAction") == 0) {
-		static GtkWidget *about = NULL;
+	gtk_main_quit ();
+}
 
-		const gchar *authors[] = {
-			"Mikael Hallendal <micke@imendio.com>",
-			"Richard Hult <richard@imendio.com>",
-			"Johan Dahlin <jdahlin@telia.com>",
-			"Ross Burton <ross@burtonini.com>",
-			NULL
-		};
+static void
+window_activate_copy (GtkAction *action, DhWindow *window)
+{
+	g_print ("Copy\n");
+}
 
-		if (!about) {
-			GtkWidget *hbox;
-			GtkWidget *href;
-			
-			about = gnome_about_new ("Devhelp", VERSION,
-						 "",
-						 _("A developer's help browser for GNOME 2"),
-						 authors,
-						 NULL,
-						 NULL,
-						 NULL);
+static void
+window_activate_preferences (GtkAction *action, DhWindow *window)
+{
+	dh_preferences_show_dialog (GTK_WINDOW (window));
+}
 
-					
-			g_signal_connect (about,
-					  "destroy",
-					  G_CALLBACK (gtk_widget_destroyed),
-					  &about);
-			hbox = gtk_hbox_new (FALSE, 0);
-			gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about)->vbox),
-					    hbox, FALSE, FALSE, 0);
-			href = gnome_href_new ("http://www.imendio.com/projects/devhelp/",
-					       _("Devhelp project page"));
-			gtk_box_pack_start (GTK_BOX (hbox), href, 
-					    TRUE, TRUE, 0);
-			href = gnome_href_new ("http://bugzilla.gnome.org/",
-					       _("Bug report Devhelp"));
-			gtk_box_pack_start (GTK_BOX (hbox), href,
-					    TRUE, TRUE, 0);
-			
-			gtk_widget_show_all (about);
-		} else {
-			gtk_window_present (GTK_WINDOW (about));
-		}
+static void window_activate_back (GtkAction *action, DhWindow *window)
+{
+	DhWindowPriv *priv;
+
+	priv = window->priv;
+
+	dh_html_go_back (priv->html);
+}
+
+static void window_activate_forward (GtkAction *action, DhWindow *window)
+{
+	DhWindowPriv *priv;
+
+	priv = window->priv;
+
+	dh_html_go_forward (priv->html);
+}
+
+static void window_activate_about            (GtkAction          *action,
+					      DhWindow           *window)
+{
+	static GtkWidget *about = NULL;
+	GtkWidget        *hbox;
+	GtkWidget        *href;
+
+	const gchar *authors[] = {
+		"Mikael Hallendal <micke@imendio.com>",
+		"Richard Hult <richard@imendio.com>",
+		"Johan Dahlin <jdahlin@telia.com>",
+		"Ross Burton <ross@burtonini.com>",
+		NULL
+	};
+	
+	if (about != NULL) {
+		gtk_window_present (GTK_WINDOW (about));
+		return;
+	}
+	
+	about = gnome_about_new ("Devhelp", VERSION,
+				 "",
+				 _("A developer's help browser for GNOME 2"),
+				 authors,
+				 NULL,
+				 NULL,
+				 NULL);
+
+	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (window));
+
+	g_signal_connect (about,
+			  "destroy",
+			  G_CALLBACK (gtk_widget_destroyed),
+			  &about);
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about)->vbox),
+			    hbox, FALSE, FALSE, 0);
+	href = gnome_href_new ("http://www.imendio.com/projects/devhelp/",
+			       _("Devhelp project page"));
+	gtk_box_pack_start (GTK_BOX (hbox), href, 
+			    TRUE, TRUE, 0);
+	href = gnome_href_new ("http://bugzilla.gnome.org/",
+			       _("Bug report Devhelp"));
+	gtk_box_pack_start (GTK_BOX (hbox), href,
+			    TRUE, TRUE, 0);
+
+	gtk_widget_show_all (about);
+}
+
+static void
+window_save_state (DhWindow *window)
+{
+	DhWindowPriv *priv;
+	GdkWindowState state;
+	gboolean       maximized;
+
+	priv = window->priv;
+
+	state = gdk_window_get_state (GTK_WIDGET (window)->window);
+	if (state & GDK_WINDOW_STATE_MAXIMIZED) {
+		maximized = TRUE;
 	} else {
-		g_message ("Unhandled action '%s'", name);
+		maximized = FALSE;
+	}
+
+	gconf_client_set_bool (gconf_client,
+			       GCONF_MAIN_WINDOW_MAXIMIZED,
+			       maximized, NULL);
+
+	/* If maximized don't save the size and position */
+	if (!maximized) {
+		int width, height;
+		int x, y;
+
+		gtk_window_get_size (GTK_WINDOW (window), &width, &height);
+		gconf_client_set_int (gconf_client,
+				      GCONF_MAIN_WINDOW_WIDTH,
+				      width, NULL);
+		gconf_client_set_int (gconf_client,
+				      GCONF_MAIN_WINDOW_HEIGHT,
+				      height, NULL);
+
+		gtk_window_get_position (GTK_WINDOW (window), &x, &y);
+		gconf_client_set_int (gconf_client,
+				      GCONF_MAIN_WINDOW_POS_X,
+				      x, NULL);
+		gconf_client_set_int (gconf_client,
+				      GCONF_MAIN_WINDOW_POS_Y,
+				      y, NULL);
+	}
+
+	gconf_client_set_int (gconf_client,
+			      GCONF_PANED_LOCATION,
+			      gtk_paned_get_position (GTK_PANED (priv->hpaned)),
+			      NULL);
+}
+
+static void
+window_restore_state (DhWindow *window)
+{
+	gboolean maximized;
+	int      width, height;
+	int      x, y;
+
+	width = gconf_client_get_int (gconf_client,
+				      GCONF_MAIN_WINDOW_WIDTH,
+				      NULL);
+
+	if (width <= 0) {
+		width = DEFAULT_WIDTH;
+	}
+
+	height = gconf_client_get_int (gconf_client,
+				       GCONF_MAIN_WINDOW_HEIGHT,
+				       NULL);
+
+	if (height <= 0) {
+		height = DEFAULT_HEIGHT;
+	}
+
+	gtk_window_set_default_size (GTK_WINDOW (window), 
+				     width, height);
+
+	x = gconf_client_get_int (gconf_client,
+				  GCONF_MAIN_WINDOW_POS_X,
+				  NULL);
+	y = gconf_client_get_int (gconf_client,
+				  GCONF_MAIN_WINDOW_POS_Y,
+				  NULL);
+
+	gtk_window_move (GTK_WINDOW (window), x, y);
+
+	maximized = gconf_client_get_bool (gconf_client,
+					   GCONF_MAIN_WINDOW_MAXIMIZED,
+					   NULL);
+	if (maximized) {
+		gtk_window_maximize (GTK_WINDOW (window));
 	}
 }
 
@@ -451,7 +583,9 @@ window_delete_cb (GtkWidget   *widget,
 {
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (DH_IS_WINDOW (widget));
-	
+
+	window_save_state (DH_WINDOW (widget));
+
 	gtk_main_quit ();
 }
 
@@ -468,6 +602,8 @@ window_open_url (DhWindow *window, const gchar *url)
 	dh_html_open_uri (priv->html, url);
 	dh_book_tree_show_uri (DH_BOOK_TREE (priv->book_tree), url);
 
+	window_check_history (window);
+	
 	return TRUE;
 }
 
@@ -481,20 +617,17 @@ window_link_selected_cb (GObject *ignored, DhLink *link, DhWindow *window)
 	
 	priv = window->priv;
 
-	if (window_open_url (window, link->uri)) {
-		dh_history_goto (priv->history, link->uri);
-	}
+	window_open_url (window, link->uri);
 }
 
 static void
-window_merge_add_widget (EggMenuMerge *merge,
+window_manager_add_widget (GtkUIManager *manager,
 			 GtkWidget    *widget,
 			 DhWindow     *window)
 {
 	DhWindowPriv *priv;
 	
 	g_return_if_fail (DH_IS_WINDOW (window));
-
 	priv = window->priv;
 
 	gtk_box_pack_start (GTK_BOX (priv->menu_box), widget,
@@ -503,44 +636,6 @@ window_merge_add_widget (EggMenuMerge *merge,
 	gtk_widget_show (widget);
 }
 
-static void
-window_back_exists_changed_cb (DhHistory *history, 
-			       gboolean   exists,
-			       DhWindow  *window)
-{
-	DhWindowPriv *priv;
-	EggAction *action;
-		
-	g_return_if_fail (DH_IS_HISTORY (history));
-	g_return_if_fail (DH_IS_WINDOW (window));
-	
-	priv = window->priv;
-	
-	action = egg_action_group_get_action (priv->action_group, 
-					      "BackAction");
-	
-	g_object_set (action, "sensitive", exists, NULL);
-}
-
-static void
-window_forward_exists_changed_cb (DhHistory *history, 
-				  gboolean   exists, 
-				  DhWindow  *window)
-{
-	DhWindowPriv *priv;
-	EggAction *action;
-		
-	g_return_if_fail (DH_IS_HISTORY (history));
-	g_return_if_fail (DH_IS_WINDOW (window));
-	
-	priv = window->priv;
-	
-	action = egg_action_group_get_action (priv->action_group, 
-					      "ForwardAction");
-	
-	g_object_set (action, "sensitive", exists, NULL);
-}
-
 static gboolean
 window_key_press_event_cb (GtkWidget   *widget,
 			   GdkEventKey *event,
@@ -560,6 +655,33 @@ window_key_press_event_cb (GtkWidget   *widget,
 	return FALSE;
 }
 
+static void
+window_check_history (DhWindow *window)
+{
+	DhWindowPriv *priv;
+	GtkAction *action;
+		
+	priv = window->priv;
+	
+	action = gtk_action_group_get_action (priv->action_group, 
+					      "Forward");
+	
+	g_object_set (action, "sensitive", 
+		      dh_html_can_go_forward (priv->html), NULL);
+	action = gtk_action_group_get_action (priv->action_group,
+					      "Back");
+	g_object_set (action, "sensitive",
+		      dh_html_can_go_back (priv->html), NULL);
+}
+
+static void
+window_location_changed_cb (DhHtml      *html,
+			    const gchar *location, 
+			    DhWindow    *window)
+{
+	window_check_history (window);
+}
+
 GtkWidget *
 dh_window_new (DhBase *base)
 {
@@ -574,20 +696,19 @@ dh_window_new (DhBase *base)
 
         gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);
 	gtk_window_set_title (GTK_WINDOW (window), "Devhelp");
-	gtk_window_set_wmclass (GTK_WINDOW (window), "devhelp", "devhelp");
 
 	if (geometry) {
 		gtk_window_parse_geometry (GTK_WINDOW (window), geometry);
 	} else {
-		gtk_window_set_default_size (GTK_WINDOW (window), 700, 500);
+		window_restore_state (window);
 	}
 
 	g_signal_connect (window, 
 			  "delete_event",
 			  G_CALLBACK (window_delete_cb),
 			  NULL);
-	
-        window_populate (window);
+
+	window_populate (window);
 
 	icon = gdk_pixbuf_new_from_file (DATA_DIR "/pixmaps/devhelp.png", 
 					 NULL);
@@ -595,10 +716,25 @@ dh_window_new (DhBase *base)
 		gtk_window_set_icon (GTK_WINDOW (window), icon);
 		g_object_unref (icon);
 	}
-	
+
 	return GTK_WIDGET (window);
 }
 
+void
+dh_window_show (DhWindow *window)
+{
+	gtk_widget_show_all (GTK_WIDGET (window));
+
+	/* Make sure that the HTML widget is realized before trying to 
+	 * clear it. Solves bug #147343.
+	 */
+	while (g_main_context_pending (NULL)) {
+		g_main_context_iteration (NULL, FALSE);
+	}
+
+	dh_html_clear (window->priv->html);
+}
+
 void
 dh_window_search (DhWindow *window, const gchar *str)
 {
